import {} from "../exp/index.js";
import { substitutionBindings } from "../substitution/Substitution.js";
export function formatExp(exp) {
    switch (exp.kind) {
        case "Var": {
            return exp.name;
        }
        case "Fn": {
            const { names, ret } = formatFn([exp.name], exp.ret);
            return `(lambda (${names.join(" ")}) ${ret})`;
        }
        case "FnRec": {
            const { names, ret } = formatFn([exp.name], exp.ret);
            return `(lambda (${names.join(" ")}) ${ret})`;
        }
        case "Ap": {
            const { target, args } = formatAp(exp.target, [formatExp(exp.arg)]);
            return `(${target} ${args.join(" ")})`;
        }
        case "Let": {
            const bindings = substitutionBindings(exp.substitution).map(formatBinding);
            return `(let (${bindings.join(" ")}) ${formatExp(exp.body)})`;
        }
    }
}
function formatFn(names, ret) {
    if (ret.kind === "Fn") {
        return formatFn([...names, ret.name], ret.ret);
    }
    else {
        return { names, ret: formatExp(ret) };
    }
}
function formatAp(target, args) {
    if (target.kind === "Ap") {
        return formatAp(target.target, [formatExp(target.arg), ...args]);
    }
    else {
        return { target: formatExp(target), args };
    }
}
function formatBinding(binding) {
    return `[${binding.name} ${formatExp(binding.exp)}]`;
}
//# sourceMappingURL=formatExp.js.map